"""Sebureem Setup webinstaller

This specific webapp is designed to fire a webserver allowing the Sebureem
administrator to setup its instance.

The command to start the webapp is ``> sebureem --init``. Webapp is served
at ``localhost`` on port ``8080``. 
"""
import os
import bcrypt
from pathlib import Path
from configparser import ConfigParser
from flask import Blueprint, render_template, flash, request
from peewee import SqliteDatabase, OperationalError

from sebureem import db, config
from sebureem.models import Sebura, Sebuks

__all__ = ["setup_app"]

setup_app = Blueprint("setup", __name__,
        url_prefix="/install", template_folder="templates")

def config_dir():
    """Return the config path depending on the server OS
    """
    if os.name == "posix":
        config_path = Path(
            os.path.expandvars('$XDG_CONFIG_HOME/sebureem')
        )
    elif os.name == "nt":
        config_path = Path(
            os.path.expandvars('%LOCALAPPDATA%/sebureem')
        )
    return config_path

@setup_app.route('/')
def index():
    """Simply return the startpage of the setup webapp
    """
    return render_template("setup/index.html")

@setup_app.route('/config/')
def create_config():
    """Create the Sebureem server config file
    """
    try:
        config_path = config_dir()
        config_path.mkdir(mode=0o755, parents=True, exist_ok=True)
        config_file = config_path / 'sebureem.ini'
        if config_file.exists():
            raise FileExistsError
        print(str(config_file))
        with open(str(config_file), "w") as f:
            f.write("# Sebureem config file")
        flash("Success: Config file created", "success")
    except FileExistsError:
        flash("Warning: Config file found", "error")
        return render_template("setup/config.html",
            config_path=config_path / 'sebureem.ini',
            override_form=True)

    return render_template("setup/config.html",
        config_path=config_file, override_form=False)

@setup_app.route('/database/', methods=['GET', 'POST'])
def create_db():
    """Allow user to set up database

    Allow user select path for the Sebureem SQLite database then create it.
    A flash message confirm a successful creation.
    """
    if request.method == 'GET':
        if os.name == "posix":
            db_default_path = os.path.expandvars('$XDG_DATA_HOME/sebureem')
        elif os.name == "nt":
            db_default_path = os.path.expandvars('%LOCALAPPDATA%\sebureem')
        return render_template("setup/database.html",
            database_path=db_default_path,
            created=False)

    if request.method == 'POST':
        db_path = Path(request.form['db_path'])
        db_path = db_path / request.form['db_name']

        try:
            db.init(str(db_path))
            db.connect()
            db.create_tables([Sebura, Sebuks])
            db.close()
            flash("Success: Database created", "success")
        except OperationalError:
            flash("Warning: Database already present at this location", "error")

        config['DATABASE'] = { 
            'path': db_path
        }
        config_path = config_dir()
        config_file = config_path / 'sebureem.ini'
        with open(str(config_file), 'w') as conf:
            config.write(conf)
        return render_template("setup/database.html",
            database_path=str(db_path),
            created=True)

@setup_app.route('/admin/', methods=['GET', 'POST'])
def create_admin():
    """Allow user to set up admin credentials

    Allow the user to set up credentials to access the Sebureem Admin Webpanel.
    Login is written in plain text, but password is hashed using bcrypt
    algorythm.
    """
    if request.method == 'GET':
        return render_template("setup/admin.html",
            created=False)

    if request.method == 'POST':
        admin_login = request.form['admin_login']
        admin_passwd = bcrypt.hashpw(
            bytes(request.form['admin_passwd'], 'utf-8'), 
            bcrypt.gensalt()
        )
        config['ADMIN'] = {
            'login': admin_login,
            'passwd': admin_passwd.decode('utf-8')
        }
        config_path = config_dir()
        config_file = config_path / 'sebureem.ini'
        with open(str(config_file), 'w') as conf:
            config.write(conf)
        flash("Success: Admin account created", "success")
        return render_template("setup/admin.html",
            created=True)

@setup_app.route('/site/', methods=['GET', 'POST'])
def create_site():
    """Allow user to set up a fist site for Sebureem

    Create a first site for Sebureem to accept comment from.
    """
    if request.method == 'GET':
        return render_template("setup/site.html")

    if request.method == 'POST':
        site_name = request.form['site_name'] 
        site_url = request.form['site_url']
        config[site_name] = {
            'url': site_url
        }
        config_path = config_dir()
        config_file = config_path / 'sebureem.ini'
        with open(str(config_file), 'w') as conf:
            config.write(conf)
        flash("Success: Site configured")
        return render_template("setup/site.html",
            created=True)



"""App

The Sebureem web-server based on Bottle

"""
from flask import url_for, redirect, request, render_template, json
from playhouse.shortcuts import model_to_dict

from sebureem import app, db
from sebureem.models import Sebura, Sebuks


@app.route('/comments/<topic>', methods=['GET', 'POST'])
def comment(topic):
    if request.method == 'POST':
        print(request.form)
        comment_text = request.form['text']
        print("Adding comment to topic {} : {}".format(topic, comment_text))

        db.connect()
        sebuks, created = Sebuks.get_or_create(name=topic)
        Sebura.create(
                topic=sebuks,
                text=comment_text
                )
        db.close()

    db.connect()
    sebureem = Sebura.select().join(Sebuks).where(Sebuks.name == topic)
    db.close()

    return render_template('sebureem.html', topic=topic, comments=sebureem)


# Routes for managing comments
@app.route('/api/<topic>', methods=['GET'])
def get_comments(topic):
    """Get all comments for a given subject
    """
    print("Fetching comments for topic {}".format(topic))

    db.connect()
    sebuks = Sebuks.get(Sebuks.name == topic)
    db.close()

    return json.jsonify(model_to_dict(sebuks, backrefs=True))


@app.route('/api/<topic>', methods=['POST'])
def post_comment(topic):
    """Post a comment to a given subject
    """
    print(request.form)
    comment_text = request.form['text']
    print("Adding comment to topic {} : {}".format(topic, comment_text))

    db.connect()
    sebuks = Sebuks.get(Sebuks.name == topic)
    Sebura.create(
        topic=sebuks,
        text=comment_text
    )
    db.close()
    return redirect(url_for('get_comments', topic=topic))


@app.route('/api/<topic>/<id>', methods=['GET'])
def edit_comment(topic, id):
    """Edit a given comment
    """
    return redirect(url_for('get_comments', topic=topic))


@app.route('/api/<topic>/<id>', methods=['GET'])
def delete_comment(topic, id):
    """Delete a given comment
    """
    return redirect(url_for('get_comments', topic=topic))



"""Sebureem admin functionality and blueprints

This module holds the functions that runs behind the admin views and CLI.
This functions allow admins to moderate and manage comments, sites and topics.

Warning: Authentication doesn't occurs in this module, it should be handled in
the CLI and the admin webapp.
"""
from sebureem.models import Sebuks, Sebura

def toggle_lock_topic(topic_id, status):
    """Lock or unlock a topic by name or id

    Try to lock (or unlock, depending on the status value) a topic either by
    name or by id. A locked topic doesn't allow further comment to be posted 
    on.
    
    :param str topic_id: The topic name or id
    :param bool status: The new status for the locked property
    :raises Sebuks.DoesNotExist: if the topic cannot be retrieved from db
    """
    try:
        topic_id = int(topic_id)
        topic = Sebuks.get(Sebuks.id == topic_id)
    except ValueError:
        topic = Sebuks.get(Sebuks.name == topic_id)

    topic.locked = status
    topic.save()

def toggle_publish_comments(comments_id, status):
    """Publish or hide a comment or a range of comments

    Try to publish or unpublish a single comment or a group of comments.
     
    :param list comments_id: The list of comment's id
    :param bool status: The new status for the comments list
    """
    comments = Sebura.select().where(Sebura.id << comments_id)
    for comment in comments:
        comment.published = status
        comment.save()

def add_site():
    """Add a new site to the database
    """
    pass

def remove_site():
    """Remove a site either by name or by id
    """
    pass



"""Sebureem Command Line Utilities

Usage:
    sebureem --init
    sebureem serve [--debug] [<host>] [<port>]
    sebureem admin (-l | -a | -p) <item>

General options:
    --debug         Start the bottle app in debug mode.
    --init          Initialize database.
    -h, --help       Show this screen.
    --version       Show version.

Admin options:
    -l, --lock          Lock specified topic, 
                        <item> must be a topic name or id.
    -u, --unlock        Unlock specified topic, 
                        <item> must be a topic name or id.
    -a, --add-site      Add a new site interactively,
                        <item> must be the site name.
    -r, --remove-site   Remove a site interactively,
                        <item> must be the site name or id.
    -p, --publish       Choose submitted comments to publish, 
                        <item> must be a topic name or id.
"""
import os
from docopt import docopt
from flask import Flask

from sebureem import app, db, config
from sebureem.models import Sebura, Sebuks
from sebureem.setup_app import setup_app


def sebureem():
    args = docopt(__doc__, version='0.0.1')

    if args['--init']:
        init_sebureem()
    elif args['serve']:
        serve(
            args['<host>'] or "localhost",
            args['<port>'] or "8080",
            args['--debug']
        )
    elif args['admin']:
        if args['-l']:
            print("Locking topic ".format(args['<item>']))
        elif args['-a']:
            print("Adding site ".format(args['<item>']))
        elif args['-r']:
            print("Removing site".format(args['<item>']))
        elif args['-p']:
            print("Publishing comments".format(args['<item>']))


def init_sebureem():
    """Serve the Sebureem setup webinstaller

    Installer runs on
    http://localhost:8080/install
    """
    app = Flask(__name__)
    app.register_blueprint(setup_app)
    app.secret_key = "sebureem_setup_secret"
    app.run('localhost', 8080, debug=True)


def serve(host='localhost', port=8080, debug=False):
    if debug:
        app.run(host, int(port), debug=True)
    else:
        app.run(host, int(port))



"""Sebureem

Sebureem is a comment server similar to Discuss.
Is purpose is to allow add easily comment sections to web pages.

Sebureem is powered by Bottle for the webserver and use Peewee ORM and SQLite 
for handling data persistence.

"Sebureem" is the Kotava word for "comments" or "group of comments"

"""
import os
import configparser
from flask import Flask
from peewee import SqliteDatabase

__version__ = "0.2.1"

app = Flask(__name__)

config = configparser.ConfigParser()
db = SqliteDatabase(None)
if os.name == "posix":
    config_path = os.path.expandvars('$XDG_CONFIG_HOME/sebureem/sebureem.ini')
elif os.name == "nt":
    config_path = os.path.expandvars('%LOCALAPPDATA%/sebureem/sebureem.ini')

try:
    config.read_file(
        open(config_path)
    )
    db.init(config['DATABASE']['path'])
except (FileNotFoundError, KeyError):
    print("Warning: Sebureem doesn't have config file.")
    print("Please fix it with the `> sebureem --init` command.")

import sebureem.views



"""Models
"""
from datetime import datetime
from peewee import Model
from peewee import CharField, TextField, BooleanField
from peewee import DateTimeField, ForeignKeyField
from sebureem import db

__all__ = ['Sebuks', 'Sebura']


class BaseModel(Model):

    class Meta:
        database = db


class Sebusik(BaseModel):
    pass


class Sebuks(BaseModel):
    name = CharField()
    locked = BooleanField(default=False)


class Sebura(BaseModel):
    text = TextField()
    date = DateTimeField(default=datetime.now())
    topic = ForeignKeyField(Sebuks, related_name='comments')
    published = BooleanField(default=False)



from setuptools import setup, find_packages

import sebureem

setup(
    name='sebureem',
    version=sebureem.__version__,
    description="A simple comment server",
    long_description=open('README.rst', encoding='utf-8').read(),
    url='https://framagit.org/Erwhann-Rouge/sebureem',
    author='Guilhem "Erwhann-Rouge" MAS-PAITRAULT',
    author_email="guilhem.mas-paitrault@protonmail.com",
    license='BSD-3-Clause/CECILL-B',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],
    keywords='sebureem comments disqus',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Flask>=0.12',
        'docopt>=0.6',
        'peewee>=2.10',
        'bcrypt>=3.1',
    ],
    entry_points={
        'console_scripts': [
            'sebureem=sebureem.cli:sebureem',
        ],
    },
)
